%% tclp__files.pl
%% Copyright (C) 2001,2002 INRIA Emmanuel Coquery

%% This library is free software; you can redistribute it and/or
%% modify it under the terms of the GNU Lesser General Public
%% License as published by the Free Software Foundation; either
%% version 2.1 of the License, or (at your option) any later version.

%% This library is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%% Lesser General Public License for more details.

%% You should have received a copy of the GNU Lesser General Public
%% License along with this library; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                                           %%
%%                                  TCLP                                     %%
%%                                                                           %%
%%         Emmanuel Coquery, projet Contraintes, INRIA Rocquencourt          %%
%%                                                                           %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% $Header: /home/beaune/contraintes/coquery/cvs/contraintes/tclpchr/src/tclp/tclp__files.pl,v 1.1 2002/10/17 15:30:18 coquery Exp $

:- module(tclp__files,[ read_file/2, load_file/1,
                        load_from_lines/2,
                        read_from_lines/3,
                        add_include/1,
                        chop_ext/3,
                        print_file/1,
                        tclp__directory/1
                      ]).

:- use_module(library(lists)).
:- use_module(library(charsio)).
:- use_module(tclp(tclp__utils)).
:- use_module(tclp(tclp__reader)).
:- use_module(tclp(tclp__errors)).
:- use_module(tclp(tclp__handle_goals)).
:- use_module(library(system)).

%% Verbose mode : debug_files

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File names
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% We add the possibility to have including directories
%% Include dirs are stored in a dynamic predicate

:- dynamic include_dir/1.

%include_dir('../').

%% The predicate to add a directory to include dirs
add_include(X) :-
        absolute_file_name(X,Y),
        assertz(include_dir(Y)).

%% last_in_list(X,List) is true if X can be unified with the last element of
%% list
last_in_list(X,[X]) :- !.
last_in_list(X,[_|Rest]) :- last_in_list(X,Rest).

%% concat_filename(DirName, FileName, CompleteFileName)
%% CompleteFileName is DirName/FileName with any trailing / removed from
%% DirName
concat_filename(DirName, FileName, CompleteFileName) :-
        atom_concat(_,'/',DirName),!, 
        atom_concat(DirName, FileName, CompleteFileName).
concat_filename(DirName, FileName, CompleteFileName) :-
        atom_concat(DirName,'/', CDirName),
        atom_concat(CDirName,FileName, CompleteFileName).

%% Enumerate possible file names
filename(FileName, CompleteFileName) :-
        FileName = CompleteFileName ;
        include_dir(Dir),
        concat_filename(Dir, FileName, CompleteFileName).

%% chop_ext(FileName, Ext, Chopped)
%% true if FileName ends with Ext and ChoppedExt = FileName
chop_ext(FileName, Ext, Chopped) :-
        atom_concat(Chopped, Ext, FileName).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File reading / loading
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% open the file by looking into include directories
open_stream(FileName, Stream) :-
        (
          FileName == '-'
        ->
          Stream = user_input
        ;
          (
            nofileerrors,
            filename(FileName, CompleteFileName),
            verbose(debug_files,['Trying to open ', CompleteFileName]),
            open(CompleteFileName, read, Stream),!
          ;
            print_error([ 'Can''t open "', FileName, '"' ])
          )
        ),
        verbose(debug_files,'stream openned').

print_file(FileName) :-
        open_stream(FileName, Stream),
        repeat,
        get_code(Stream,Code),
        (
          Code = -1
        ->
          true
        ;
          put_code(Code),
          fail
        ),!,
        close(Stream).

%% Read a stream : read each phrase of the stream, accumulating lines number
%% and variable names.
%% also executes directives when needed
read_stream(FileName, Stream, PhraseList) :-
        read_one_term(Stream, Term, TermData, FileName),
        (
          Term == end_of_file
        ->
          PhraseList = []
        ;
          handle_term(Term,TermData),
          PhraseList = [ Term-TermData | Rest ],
          read_stream(FileName, Stream, Rest)
        )
        ;
        print_error('failed to read a term').

%% Read a file
read_file(FileName, PhraseList) :-
        open_stream(FileName, Stream),
        read_stream(FileName, Stream, PhraseList),
        close(Stream),!.

%% Read a list of character codes
read_codes(FileName, CharCodeList, PhraseList) :-
        open_chars_stream(CharCodeList, Stream),
        read_stream(FileName, Stream, PhraseList),
        close(Stream),!.

%% Reads a list of lines
read_from_lines(FileName, LineList, PhraseList) :-
        string_list_to_character_code_list(LineList, CodeList),
        read_codes(FileName,CodeList,PhraseList).

:- dynamic loaded/1.

%% Loads a stream: same as read except that the phrases are left over
load_stream(FileName, _) :- loaded(FileName),!.
load_stream(FileName, Stream) :-
        read_stream(FileName, Stream,_).

%% Loads a file: same as read except that the phrases are left over
load_file(FileName) :- loaded(FileName),!.
load_file(FileName) :-
        assertz(loaded(FileName)),
        read_file(FileName, _).

%% Loads a list of char codes
load_codes(FileName, _) :- loaded(FileName),!.
load_codes(FileName, CharCodeList) :-
        assertz(loaded(FileName)),
        read_codes(FileName, CharCodeList, _).

%% Loads a list of lines
load_from_lines(FileName, _) :- loaded(FileName),!.
load_from_lines(FileName, LineList) :-
        assertz(loaded(FileName)),
        read_from_lines(FileName, LineList, _).

%% reinits the loaded files and the include dirs
reinit :-
        retractall(loaded(_)),
        retractall(include_dir(_)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% TCLP directory
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tclp__directory(Dir) :-
        prolog_load_context(directory,Dir) -> true ;
        environ('SP_APP_DIR',Dir) -> true;
        Dir = unknown.
